//+------------------------------------------------------------------+
//|                                                    dmx - oma.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    5
#property indicator_color1     DarkSlateGray
#property indicator_color2     DarkSlateGray
#property indicator_color3     DeepSkyBlue
#property indicator_color4     Red
#property indicator_color5     DimGray
#property indicator_style1     STYLE_DOT
#property indicator_style2     STYLE_DOT
#property indicator_levelcolor DarkSlateGray
#property indicator_level1     0
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern int    Length          = 32;
extern double Speed           = 2.5;
extern bool   Adaptive        = false;
extern int    SmoothLength    = 5;
extern bool   SmoothAdaptive  = false;
extern double SmoothSpeed     = 3.0;
extern bool   ShowHistogram   = true;
extern bool   ShowUpDownLimit = true;

//
//
//
//
//

double dmx[];
double dmxU[];
double dmxD[];
double limU[];
double limD[];
double stored[][28];



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,limU); SetIndexLabel(0,NULL);
   SetIndexBuffer(1,limD); SetIndexLabel(1,NULL);
   SetIndexBuffer(4,dmx);  SetIndexLabel(4,"Dmx");
      if (ShowHistogram)
      {
         SetIndexBuffer(2,dmxU); SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexLabel(2,"Dmx");
         SetIndexBuffer(3,dmxD); SetIndexStyle(3,DRAW_HISTOGRAM); SetIndexLabel(3,"DmX");
      }
      else
      {
         SetIndexStyle(2,DRAW_NONE); SetIndexLabel(2,NULL);
         SetIndexStyle(3,DRAW_NONE); SetIndexLabel(3,NULL);
      }
   SmoothLength =MathMax(SmoothLength,1);

   //
   //
   //
   //
   //
   
   IndicatorShortName("Dmx oma("+Length+","+DoubleToStr(Speed,Digits)+","+SmoothLength+","+DoubleToStr(SmoothSpeed,Digits)+")");
   return(0);
}
int deinit() { return(0); }




//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=Bars-counted_bars;
           if (ArrayRange(stored,0) != Bars) ArrayResize(stored,Bars);

   //
   //
   //
   //
   //
   
   for(i=limit, r=Bars-limit-1; i>=0; i--,r++)
   {
      double currTR  = iAverage(MathMax(High[i],Close[i+1])-MathMin(Low[i],Close[i+1]),Length,Speed,Adaptive,r,0);
      double DeltaHi = High[i] - High[i+1];
      double DeltaLo = Low[i+1] - Low[i];
      double plusDM  = 0.00;
      double minusDM = 0.00;

         if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
         if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
      //
      //
      //
      //
      //

         double DIp = 0.00;
         double DIm = 0.00;

            if (currTR > 0.00)
            {
               DIp = 100.0*iAverage(plusDM ,Length,Speed,false,r,7)/currTR;
               DIm = 100.0*iAverage(minusDM,Length,Speed,false,r,14)/currTR;
            }
            if ((DIp+DIm) != 0)
                  dmx[i] = 100.00*iAverage((DIp-DIm)/(DIp+DIm),SmoothLength,SmoothSpeed,SmoothAdaptive,r,21);
            else  dmx[i] = 100.00*iAverage(0                  ,SmoothLength,SmoothSpeed,SmoothAdaptive,r,21);
         
      //
      //
      //
      //
      //
               
      if (ShowHistogram)
      {
         dmxD[i] = dmxD[i+1];
         dmxU[i] = dmxU[i+1];
            if (dmx[i] > dmx[i+1]) { dmxU[i] = dmx[i];  dmxD[i] = EMPTY_VALUE; }
            if (dmx[i] < dmx[i+1]) { dmxD[i] = dmx[i];  dmxU[i] = EMPTY_VALUE; }
      }
      if (ShowUpDownLimit)
      {
         limD[i] = -100;
         limU[i] =  100;
      }         
   }   

   //
   //
   //
   //
   //
   
   return(0);      
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define E1  0
#define E2  1
#define E3  2
#define E4  3
#define E5  4
#define E6  5
#define res 6

//
//
//
//
//

double iAverage(double price, double averagePeriod, double tconst, bool adaptive, int r, int ashift=0)
{
   double e1=stored[r-1][E1+ashift];  double e2=stored[r-1][E2+ashift];
   double e3=stored[r-1][E3+ashift];  double e4=stored[r-1][E4+ashift];
   double e5=stored[r-1][E5+ashift];  double e6=stored[r-1][E6+ashift];

   //
   //
   //
   //
   //

      if (adaptive && (averagePeriod > 1))
      {
         double minPeriod = averagePeriod/2.0;
         double maxPeriod = minPeriod*5.0;
         int    endPeriod = MathCeil(maxPeriod);
         double signal    = MathAbs((price-stored[r-endPeriod][res]));
         double noise     = 0.00000000001;

            for(int k=1; k<endPeriod; k++) noise=noise+MathAbs(price-stored[r-k][res]);

         averagePeriod = ((signal/noise)*(maxPeriod-minPeriod))+minPeriod;
      }
      
      //
      //
      //
      //
      //
      
      double alpha = (2.0+tconst)/(1.0+tconst+averagePeriod);

      e1 = e1 + alpha*(price-e1); e2 = e2 + alpha*(e1-e2); double v1 = 1.5 * e1 - 0.5 * e2;
      e3 = e3 + alpha*(v1   -e3); e4 = e4 + alpha*(e3-e4); double v2 = 1.5 * e3 - 0.5 * e4;
      e5 = e5 + alpha*(v2   -e5); e6 = e6 + alpha*(e5-e6); double v3 = 1.5 * e5 - 0.5 * e6;

   //
   //
   //
   //
   //

   stored[r][E1+ashift]  = e1;  stored[r][E2+ashift] = e2;
   stored[r][E3+ashift]  = e3;  stored[r][E4+ashift] = e4;
   stored[r][E5+ashift]  = e5;  stored[r][E6+ashift] = e6;
   stored[r][res+ashift] = price;
   return(v3);
}